# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-present Noritem

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from .mixins import Hashable
from .utils import parse_time
from .enums import EntitlementType, try_enum

__all__ = (
    'Entitlement',
)


class Entitlement(Hashable):
    """Represents a Discord entitlement.

    Attributes
    ----------
    id: :class:`int`
        The entitlement's ID.
    sku_id: :class:`int`
        The ID of the SKU.
    application_id: :class:`int`
        The ID of the parent application.
    user_id: Optional[:class:`int`]
        The user ID granted access.
    type: :class:`EntitlementType`
        The type of entitlement.
    deleted: :class:`bool`
        Whether the entitlement was deleted.
    starts_at: Optional[:class:`datetime.datetime`]
        When the entitlement is valid from.
    ends_at: Optional[:class:`datetime.datetime`]
        When the entitlement is no longer valid.
    guild_id: Optional[:class:`int`]
        The guild ID granted access.
    consumed: :class:`bool`
        Whether the consumable entitlement has been consumed.
    """

    __slots__ = (
        '_state', 'id', 'sku_id', 'application_id', 'user_id',
        'type', 'deleted', 'starts_at', 'ends_at', 'guild_id', 'consumed',
    )

    def __init__(self, *, state=None, data):
        self._state = state
        self.id = int(data['id'])
        self.sku_id = int(data['sku_id'])
        self.application_id = int(data['application_id'])
        user_id = data.get('user_id')
        self.user_id = int(user_id) if user_id else None
        self.type = try_enum(EntitlementType, data.get('type', 1))
        self.deleted = data.get('deleted', False)
        self.starts_at = parse_time(data.get('starts_at'))
        self.ends_at = parse_time(data.get('ends_at'))
        guild_id = data.get('guild_id')
        self.guild_id = int(guild_id) if guild_id else None
        self.consumed = data.get('consumed', False)

    def __repr__(self):
        return '<Entitlement id={0.id} sku_id={0.sku_id} type={0.type!r}>'.format(self)

    @property
    def is_active(self):
        """:class:`bool`: Whether this entitlement is currently active (not deleted and not ended)."""
        return not self.deleted

    async def consume(self):
        """|coro|

        Marks a consumable entitlement as consumed.
        """
        if self._state:
            await self._state.http.consume_entitlement(self.application_id, self.id)
            self.consumed = True



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-present Noritem

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from .mixins import Hashable
from .enums import (
    AutoModTriggerType,
    AutoModEventType,
    AutoModActionType,
    AutoModKeywordPresetType,
    try_enum,
)

__all__ = (
    'AutoModRule',
    'AutoModAction',
)


class AutoModAction:
    """Represents an auto moderation action.

    Attributes
    ----------
    type: :class:`AutoModActionType`
        The type of action.
    metadata: :class:`dict`
        Additional metadata for the action.
    """

    __slots__ = ('type', 'metadata')

    def __init__(self, data):
        self.type = try_enum(AutoModActionType, data['type'])
        self.metadata = data.get('metadata', {})

    def to_dict(self):
        d = {'type': self.type.value}
        if self.metadata:
            d['metadata'] = self.metadata
        return d

    def __repr__(self):
        return '<AutoModAction type={0.type!r}>'.format(self)


class AutoModRule(Hashable):
    """Represents an auto moderation rule.

    Attributes
    ----------
    id: :class:`int`
        The rule's ID.
    guild_id: :class:`int`
        The guild ID.
    name: :class:`str`
        The rule name.
    creator_id: :class:`int`
        The ID of the user who created the rule.
    event_type: :class:`AutoModEventType`
        The event type.
    trigger_type: :class:`AutoModTriggerType`
        The trigger type.
    trigger_metadata: :class:`dict`
        The trigger metadata.
    actions: List[:class:`AutoModAction`]
        The actions to execute when the rule triggers.
    enabled: :class:`bool`
        Whether the rule is enabled.
    exempt_roles: List[:class:`int`]
        Role IDs exempt from the rule.
    exempt_channels: List[:class:`int`]
        Channel IDs exempt from the rule.
    """

    __slots__ = (
        '_state', 'id', 'guild_id', 'name', 'creator_id', 'event_type',
        'trigger_type', 'trigger_metadata', 'actions', 'enabled',
        'exempt_roles', 'exempt_channels',
    )

    def __init__(self, *, state, data):
        self._state = state
        self._update(data)

    def _update(self, data):
        self.id = int(data['id'])
        self.guild_id = int(data['guild_id'])
        self.name = data['name']
        self.creator_id = int(data['creator_id'])
        self.event_type = try_enum(AutoModEventType, data.get('event_type', 1))
        self.trigger_type = try_enum(AutoModTriggerType, data.get('trigger_type', 1))
        self.trigger_metadata = data.get('trigger_metadata', {})
        self.actions = [AutoModAction(a) for a in data.get('actions', [])]
        self.enabled = data.get('enabled', False)
        self.exempt_roles = [int(r) for r in data.get('exempt_roles', [])]
        self.exempt_channels = [int(c) for c in data.get('exempt_channels', [])]

    def __repr__(self):
        return '<AutoModRule id={0.id} name={0.name!r} guild_id={0.guild_id} enabled={0.enabled}>'.format(self)

    async def edit(self, *, reason=None, **fields):
        """|coro|

        Edits the auto moderation rule.

        Parameters
        ----------
        name: :class:`str`
            The rule name.
        event_type: :class:`AutoModEventType`
            The event type.
        trigger_metadata: :class:`dict`
            The trigger metadata.
        actions: List[:class:`dict`]
            The actions (list of action dicts).
        enabled: :class:`bool`
            Whether the rule is enabled.
        exempt_roles: List[:class:`int`]
            Role IDs exempt from the rule.
        exempt_channels: List[:class:`int`]
            Channel IDs exempt from the rule.
        reason: Optional[:class:`str`]
            The reason for editing.
        """
        payload = {}
        for key in ('name', 'trigger_metadata', 'enabled'):
            if key in fields:
                payload[key] = fields[key]

        if 'event_type' in fields:
            payload['event_type'] = fields['event_type'].value
        if 'actions' in fields:
            payload['actions'] = fields['actions']
        if 'exempt_roles' in fields:
            payload['exempt_roles'] = [str(r) for r in fields['exempt_roles']]
        if 'exempt_channels' in fields:
            payload['exempt_channels'] = [str(c) for c in fields['exempt_channels']]

        data = await self._state.http.edit_automod_rule(self.guild_id, self.id, reason=reason, **payload)
        self._update(data)

    async def delete(self, *, reason=None):
        """|coro|

        Deletes the auto moderation rule.

        Parameters
        ----------
        reason: Optional[:class:`str`]
            The reason for deleting.
        """
        await self._state.http.delete_automod_rule(self.guild_id, self.id, reason=reason)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-present Noritem

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from .enums import PollLayoutType, try_enum

__all__ = (
    'Poll',
    'PollMedia',
    'PollAnswer',
    'PollResults',
    'PollAnswerCount',
)


class PollMedia:
    """Represents poll media (text and optional emoji).

    Attributes
    ----------
    text: Optional[:class:`str`]
        The text of the field.
    emoji: Optional[:class:`dict`]
        The emoji of the field.
    """

    __slots__ = ('text', 'emoji')

    def __init__(self, data):
        self.text = data.get('text')
        self.emoji = data.get('emoji')

    def to_dict(self):
        d = {}
        if self.text is not None:
            d['text'] = self.text
        if self.emoji is not None:
            d['emoji'] = self.emoji
        return d

    def __repr__(self):
        return '<PollMedia text={0.text!r}>'.format(self)


class PollAnswer:
    """Represents a poll answer.

    Attributes
    ----------
    answer_id: Optional[:class:`int`]
        The ID of the answer.
    poll_media: :class:`PollMedia`
        The data of the answer.
    """

    __slots__ = ('answer_id', 'poll_media')

    def __init__(self, data):
        self.answer_id = data.get('answer_id')
        self.poll_media = PollMedia(data.get('poll_media', {}))

    def to_dict(self):
        d = {}
        if self.answer_id is not None:
            d['answer_id'] = self.answer_id
        d['poll_media'] = self.poll_media.to_dict()
        return d

    def __repr__(self):
        return '<PollAnswer answer_id={0.answer_id} poll_media={0.poll_media!r}>'.format(self)


class PollAnswerCount:
    """Represents a vote count for a poll answer.

    Attributes
    ----------
    id: :class:`int`
        The answer_id.
    count: :class:`int`
        The number of votes.
    me_voted: :class:`bool`
        Whether the current user voted for this answer.
    """

    __slots__ = ('id', 'count', 'me_voted')

    def __init__(self, data):
        self.id = data['id']
        self.count = data['count']
        self.me_voted = data.get('me_voted', False)

    def __repr__(self):
        return '<PollAnswerCount id={0.id} count={0.count}>'.format(self)


class PollResults:
    """Represents poll results.

    Attributes
    ----------
    is_finalized: :class:`bool`
        Whether the votes have been precisely counted.
    answer_counts: List[:class:`PollAnswerCount`]
        The counts for each answer.
    """

    __slots__ = ('is_finalized', 'answer_counts')

    def __init__(self, data):
        self.is_finalized = data.get('is_finalized', False)
        self.answer_counts = [PollAnswerCount(a) for a in data.get('answer_counts', [])]

    def __repr__(self):
        return '<PollResults is_finalized={0.is_finalized} answers={1}>'.format(self, len(self.answer_counts))


class Poll:
    """Represents a poll attached to a message.

    Attributes
    ----------
    question: :class:`PollMedia`
        The question of the poll.
    answers: List[:class:`PollAnswer`]
        The answers available.
    expiry: Optional[:class:`str`]
        ISO8601 timestamp when the poll ends.
    allow_multiselect: :class:`bool`
        Whether users can select multiple answers.
    layout_type: :class:`PollLayoutType`
        The layout type of the poll.
    results: Optional[:class:`PollResults`]
        The results of the poll, if available.
    """

    __slots__ = ('_state', 'question', 'answers', 'expiry',
                 'allow_multiselect', 'layout_type', 'results',
                 '_channel_id', '_message_id')

    def __init__(self, *, state=None, data, channel_id=None, message_id=None):
        self._state = state
        self._channel_id = channel_id
        self._message_id = message_id
        self.question = PollMedia(data.get('question', {}))
        self.answers = [PollAnswer(a) for a in data.get('answers', [])]
        self.expiry = data.get('expiry')
        self.allow_multiselect = data.get('allow_multiselect', False)
        self.layout_type = try_enum(PollLayoutType, data.get('layout_type', 1))
        results = data.get('results')
        self.results = PollResults(results) if results else None

    def __repr__(self):
        return '<Poll question={0.question!r} answers={1}>'.format(self, len(self.answers))

    async def end(self):
        """|coro|

        Immediately ends the poll.
        Requires the message to be authored by the bot.
        """
        if self._state and self._channel_id and self._message_id:
            await self._state.http.end_poll(self._channel_id, self._message_id)

    async def get_voters(self, answer_id, *, after=None, limit=None):
        """|coro|

        Gets the users that voted for a specific answer.

        Parameters
        ----------
        answer_id: :class:`int`
            The answer ID to get voters for.
        after: Optional[:class:`int`]
            Get users after this user ID.
        limit: Optional[:class:`int`]
            Max number of users to return.

        Returns
        -------
        :class:`dict`
            The response data with voter information.
        """
        if self._state and self._channel_id and self._message_id:
            return await self._state.http.get_answer_voters(
                self._channel_id, self._message_id, answer_id,
                after=after, limit=limit,
            )



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-present Noritem

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from .mixins import Hashable
from .utils import snowflake_time, parse_time
from .enums import (
    ScheduledEventEntityType,
    ScheduledEventStatus,
    ScheduledEventPrivacyLevel,
    try_enum,
)

__all__ = (
    'ScheduledEvent',
    'ScheduledEventUser',
)


class ScheduledEvent(Hashable):
    """Represents a guild scheduled event.

    Attributes
    ----------
    id: :class:`int`
        The scheduled event's ID.
    guild_id: :class:`int`
        The guild ID.
    channel_id: Optional[:class:`int`]
        The channel ID, or ``None`` if entity type is ``EXTERNAL``.
    creator_id: Optional[:class:`int`]
        The user ID who created the event.
    name: :class:`str`
        The name of the event.
    description: Optional[:class:`str`]
        The description.
    scheduled_start_time: :class:`datetime.datetime`
        When the event will start.
    scheduled_end_time: Optional[:class:`datetime.datetime`]
        When the event will end.
    privacy_level: :class:`ScheduledEventPrivacyLevel`
        The privacy level.
    status: :class:`ScheduledEventStatus`
        The status.
    entity_type: :class:`ScheduledEventEntityType`
        The entity type.
    entity_id: Optional[:class:`int`]
        The entity ID.
    entity_metadata: Optional[:class:`dict`]
        Additional metadata (e.g. location for external events).
    user_count: Optional[:class:`int`]
        The number of users subscribed.
    image: Optional[:class:`str`]
        The cover image hash.
    recurrence_rule: Optional[:class:`dict`]
        The recurrence rule data.
    """

    __slots__ = (
        '_state', 'id', 'guild_id', 'channel_id', 'creator_id', 'name',
        'description', 'scheduled_start_time', 'scheduled_end_time',
        'privacy_level', 'status', 'entity_type', 'entity_id',
        'entity_metadata', 'creator', 'user_count', 'image', 'recurrence_rule',
    )

    def __init__(self, *, state, data):
        self._state = state
        self._update(data)

    def _update(self, data):
        self.id = int(data['id'])
        self.guild_id = int(data['guild_id'])
        channel_id = data.get('channel_id')
        self.channel_id = int(channel_id) if channel_id else None
        creator_id = data.get('creator_id')
        self.creator_id = int(creator_id) if creator_id else None
        self.name = data['name']
        self.description = data.get('description')
        self.scheduled_start_time = parse_time(data.get('scheduled_start_time'))
        self.scheduled_end_time = parse_time(data.get('scheduled_end_time'))
        self.privacy_level = try_enum(ScheduledEventPrivacyLevel, data.get('privacy_level', 2))
        self.status = try_enum(ScheduledEventStatus, data.get('status', 1))
        self.entity_type = try_enum(ScheduledEventEntityType, data.get('entity_type', 1))
        entity_id = data.get('entity_id')
        self.entity_id = int(entity_id) if entity_id else None
        self.entity_metadata = data.get('entity_metadata')
        self.creator = data.get('creator')
        self.user_count = data.get('user_count')
        self.image = data.get('image')
        self.recurrence_rule = data.get('recurrence_rule')

    @property
    def created_at(self):
        """:class:`datetime.datetime`: Returns the event's creation time in UTC."""
        return snowflake_time(self.id)

    @property
    def location(self):
        """Optional[:class:`str`]: The location of an external event."""
        if self.entity_metadata:
            return self.entity_metadata.get('location')
        return None

    def __repr__(self):
        return '<ScheduledEvent id={0.id} name={0.name!r} guild_id={0.guild_id} status={0.status!r}>'.format(self)

    async def edit(self, *, reason=None, **fields):
        """|coro|

        Edits the scheduled event.

        Parameters
        ----------
        name: :class:`str`
            The new name.
        description: :class:`str`
            The new description.
        channel_id: Optional[:class:`int`]
            The new channel ID.
        scheduled_start_time: :class:`str`
            ISO8601 start time.
        scheduled_end_time: :class:`str`
            ISO8601 end time.
        privacy_level: :class:`ScheduledEventPrivacyLevel`
            The privacy level.
        entity_type: :class:`ScheduledEventEntityType`
            The entity type.
        status: :class:`ScheduledEventStatus`
            The event status.
        entity_metadata: :class:`dict`
            The entity metadata.
        image: :class:`str`
            The cover image (base64-encoded).
        reason: Optional[:class:`str`]
            The reason for editing.
        """
        payload = {}
        for key in ('name', 'description', 'channel_id', 'scheduled_start_time',
                     'scheduled_end_time', 'entity_metadata', 'image', 'recurrence_rule'):
            if key in fields:
                payload[key] = fields[key]

        if 'privacy_level' in fields:
            payload['privacy_level'] = fields['privacy_level'].value
        if 'entity_type' in fields:
            payload['entity_type'] = fields['entity_type'].value
        if 'status' in fields:
            payload['status'] = fields['status'].value

        data = await self._state.http.edit_scheduled_event(self.guild_id, self.id, reason=reason, **payload)
        self._update(data)

    async def delete(self):
        """|coro|

        Deletes the scheduled event.
        """
        await self._state.http.delete_scheduled_event(self.guild_id, self.id)

    async def fetch_users(self, *, limit=100, with_member=False, before=None, after=None):
        """|coro|

        Fetches users subscribed to this event.

        Returns
        -------
        List[:class:`dict`]
            A list of scheduled event user data.
        """
        return await self._state.http.get_scheduled_event_users(
            self.guild_id, self.id,
            limit=limit, with_member=with_member,
            before=before, after=after,
        )


class ScheduledEventUser:
    """Represents a user subscribed to a scheduled event.

    Attributes
    ----------
    guild_scheduled_event_id: :class:`int`
        The event ID.
    user: :class:`dict`
        The user data.
    member: Optional[:class:`dict`]
        The guild member data, if available.
    """

    __slots__ = ('guild_scheduled_event_id', 'user', 'member')

    def __init__(self, data):
        self.guild_scheduled_event_id = int(data['guild_scheduled_event_id'])
        self.user = data['user']
        self.member = data.get('member')

    def __repr__(self):
        return '<ScheduledEventUser event_id={0.guild_scheduled_event_id}>'.format(self)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-present Noritem

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from .mixins import Hashable
from .enums import StagePrivacyLevel, try_enum

__all__ = (
    'StageInstance',
)


class StageInstance(Hashable):
    """Represents a stage instance.

    Attributes
    ----------
    id: :class:`int`
        The stage instance's ID.
    guild_id: :class:`int`
        The guild ID of the associated stage channel.
    channel_id: :class:`int`
        The ID of the associated stage channel.
    topic: :class:`str`
        The topic of the stage instance (1-120 characters).
    privacy_level: :class:`StagePrivacyLevel`
        The privacy level of the stage instance.
    discoverable_disabled: :class:`bool`
        Whether or not stage discovery is disabled.
    guild_scheduled_event_id: Optional[:class:`int`]
        The ID of the scheduled event for this stage instance.
    """

    __slots__ = (
        '_state', 'id', 'guild_id', 'channel_id', 'topic',
        'privacy_level', 'discoverable_disabled', 'guild_scheduled_event_id',
    )

    def __init__(self, *, state, data):
        self._state = state
        self._update(data)

    def _update(self, data):
        self.id = int(data['id'])
        self.guild_id = int(data['guild_id'])
        self.channel_id = int(data['channel_id'])
        self.topic = data.get('topic', '')
        self.privacy_level = try_enum(StagePrivacyLevel, data.get('privacy_level', 2))
        self.discoverable_disabled = data.get('discoverable_disabled', False)
        event_id = data.get('guild_scheduled_event_id')
        self.guild_scheduled_event_id = int(event_id) if event_id else None

    def __repr__(self):
        return '<StageInstance id={0.id} guild_id={0.guild_id} channel_id={0.channel_id} topic={0.topic!r}>'.format(self)

    async def edit(self, *, topic=None, privacy_level=None, reason=None):
        """|coro|

        Edits the stage instance.

        Parameters
        ----------
        topic: :class:`str`
            The new topic of the stage instance.
        privacy_level: :class:`StagePrivacyLevel`
            The new privacy level.
        reason: Optional[:class:`str`]
            The reason for editing. Shows up in audit logs.
        """
        payload = {}
        if topic is not None:
            payload['topic'] = topic
        if privacy_level is not None:
            payload['privacy_level'] = privacy_level.value
        data = await self._state.http.edit_stage_instance(self.channel_id, reason=reason, **payload)
        self._update(data)

    async def delete(self, *, reason=None):
        """|coro|

        Deletes the stage instance.

        Parameters
        ----------
        reason: Optional[:class:`str`]
            The reason for deleting. Shows up in audit logs.
        """
        await self._state.http.delete_stage_instance(self.channel_id, reason=reason)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-present Noritem

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from .mixins import Hashable
from .utils import snowflake_time

__all__ = (
    'SoundboardSound',
)


class SoundboardSound(Hashable):
    """Represents a soundboard sound.

    Attributes
    ----------
    name: :class:`str`
        The name of the sound.
    sound_id: :class:`int`
        The ID of the sound.
    volume: :class:`float`
        The volume of the sound (0 to 1).
    emoji_id: Optional[:class:`int`]
        The ID of the custom emoji associated with the sound.
    emoji_name: Optional[:class:`str`]
        The unicode emoji associated with the sound.
    guild_id: Optional[:class:`int`]
        The ID of the guild this sound belongs to.
    available: :class:`bool`
        Whether the sound is available.
    user: Optional
        The user who created the sound.
    """

    __slots__ = (
        '_state', 'name', 'sound_id', 'volume', 'emoji_id',
        'emoji_name', 'guild_id', 'available', 'user',
    )

    def __init__(self, *, state, data):
        self._state = state
        self._update(data)

    def _update(self, data):
        self.name = data['name']
        self.sound_id = int(data['sound_id'])
        self.volume = data.get('volume', 1.0)
        emoji_id = data.get('emoji_id')
        self.emoji_id = int(emoji_id) if emoji_id else None
        self.emoji_name = data.get('emoji_name')
        guild_id = data.get('guild_id')
        self.guild_id = int(guild_id) if guild_id else None
        self.available = data.get('available', True)
        self.user = data.get('user')

    @property
    def id(self):
        """:class:`int`: Alias for :attr:`sound_id`."""
        return self.sound_id

    @property
    def created_at(self):
        """:class:`datetime.datetime`: Returns the sound's creation time in UTC."""
        return snowflake_time(self.sound_id)

    def __repr__(self):
        return '<SoundboardSound name={0.name!r} sound_id={0.sound_id} guild_id={0.guild_id}>'.format(self)

    async def edit(self, *, name=None, volume=None, emoji_id=None, emoji_name=None, reason=None):
        """|coro|

        Edits the soundboard sound.

        Parameters
        ----------
        name: :class:`str`
            The new name.
        volume: :class:`float`
            The new volume (0 to 1).
        emoji_id: Optional[:class:`int`]
            The custom emoji ID.
        emoji_name: Optional[:class:`str`]
            The unicode emoji name.
        reason: Optional[:class:`str`]
            The reason for editing.
        """
        if self.guild_id is None:
            raise ValueError('Cannot edit a default soundboard sound.')
        payload = {}
        if name is not None:
            payload['name'] = name
        if volume is not None:
            payload['volume'] = volume
        if emoji_id is not None:
            payload['emoji_id'] = str(emoji_id)
        if emoji_name is not None:
            payload['emoji_name'] = emoji_name
        data = await self._state.http.edit_guild_soundboard_sound(self.guild_id, self.sound_id, reason=reason, **payload)
        self._update(data)

    async def delete(self, *, reason=None):
        """|coro|

        Deletes the soundboard sound.

        Parameters
        ----------
        reason: Optional[:class:`str`]
            The reason for deleting.
        """
        if self.guild_id is None:
            raise ValueError('Cannot delete a default soundboard sound.')
        await self._state.http.delete_guild_soundboard_sound(self.guild_id, self.sound_id, reason=reason)
